# -*- coding: utf-8 -*-
"""Base module for unittesting."""
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import login
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.testing import z2

import collective.eeafaceted.batchactions


class NakedPloneLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)
    products = ('collective.eeafaceted.batchactions', 'eea.facetednavigation')

    def setUpZope(self, app, configurationContext):
        """Set up Zope."""
        # Load ZCML
        self.loadZCML(package=collective.eeafaceted.batchactions,
                      name='testing.zcml')
        for p in self.products:
            z2.installProduct(app, p)

    def tearDownZope(self, app):
        """Tear down Zope."""
        pass

NAKED_PLONE_FIXTURE = NakedPloneLayer(
    name="NAKED_PLONE_FIXTURE"
)

NAKED_PLONE_INTEGRATION = IntegrationTesting(
    bases=(NAKED_PLONE_FIXTURE,),
    name="NAKED_PLONE_INTEGRATION"
)


class CollectiveEeafacetedBatchActionsLayer(NakedPloneLayer):

    def setUpPloneSite(self, portal):
        """Set up Plone."""
        # Install into Plone site using portal_setup
        applyProfile(portal, 'collective.eeafaceted.batchactions:testing')

        # Login and create some test content
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)
        # make sure we have a default workflow
        portal.portal_workflow.setDefaultChain('simple_publication_workflow')


FIXTURE = CollectiveEeafacetedBatchActionsLayer(
    name="FIXTURE"
)


INTEGRATION = IntegrationTesting(
    bases=(FIXTURE,),
    name="INTEGRATION"
)


FUNCTIONAL = FunctionalTesting(
    bases=(FIXTURE,),
    name="FUNCTIONAL"
)


ACCEPTANCE = FunctionalTesting(bases=(FIXTURE,
                                      REMOTE_LIBRARY_BUNDLE_FIXTURE,
                                      z2.ZSERVER_FIXTURE),
                               name="ACCEPTANCE")



# -*- coding: utf-8 -*-

from collective.eeafaceted.batchactions.interfaces import IBatchActionsMarker
from plone.app.layout.viewlets import ViewletBase
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getGlobalSiteManager
from zope.component import getMultiAdapter
from zope.interface import Interface


class BatchActionsViewlet(ViewletBase):
    ''' '''

    index = ViewPageTemplateFile("templates/batch_actions_viewlet.pt")

    def available(self):
        """Global availability of the viewlet."""
        return True

    def _get_marker_interfaces(self):
        """By default views are registered for the IBatchActionsMarker
           interface, but in case it is needed to register different views,
           it is possible to use a more specific marker interface that inherits
           from IBatchActionsMarker."""
        ifaces = [IBatchActionsMarker]
        # check if context is marked with an interface
        # inheriting from IBatchActionsMarker
        for iface in self.context.__provides__:
            if IBatchActionsMarker in iface.__bases__:
                ifaces.append(iface)
        return ifaces

    def get_batch_actions(self):
        """We return every views that are registered for
           IBatchActionsMarker and sub interfaces."""
        # get the marker interfaces the views are registered for
        # as the viewlet is registered for IBatchActionsMarker, we will at least
        # get this interfaces in _get_marker_interfaces
        marker_interfaces = self._get_marker_interfaces()

        # get every views registered for the marker_interfaces
        gsm = getGlobalSiteManager()
        registered_actions = [
            a for a in gsm.registeredAdapters()
            if set(marker_interfaces).intersection(set(a.required)) and
            a.provided == Interface]
        # in case a view is registered several times for different interfaces,
        # we have the same name several times and we remove duplicates.
        # When getting the view, the ZCA will do the job
        registered_actions = set([action.name for action in registered_actions])
        # now check that action is available
        actions = []
        for registered_action in registered_actions:
            form = getMultiAdapter((self.context, self.request), name=registered_action)
            if form.available():
                actions.append({
                    'name': registered_action,
                    'button_with_icon': form.button_with_icon,
                    'overlay': form.overlay})
        return actions



# -*- coding: utf-8 -*-
"""Batch actions views."""

from operator import attrgetter

from AccessControl import Unauthorized
from zope import schema
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm

from plone import api
from plone.supermodel import model
from z3c.form.form import Form
from z3c.form import button
from z3c.form.field import Fields
from z3c.form.interfaces import HIDDEN_MODE
from zope.i18n import translate
from Products.CMFPlone import PloneMessageFactory as PMF

from collective.eeafaceted.batchactions import _


class IBaseBatchActionsFormSchema(model.Schema):

    uids = schema.TextLine(
        title=u"uids",
        description=u''
    )

    referer = schema.TextLine(
        title=u'referer',
        required=False,
    )


class BaseBatchActionForm(Form):

    label = _(u"Batch action form")
    fields = Fields(IBaseBatchActionsFormSchema)
    fields['uids'].mode = HIDDEN_MODE
    fields['referer'].mode = HIDDEN_MODE
    ignoreContext = True
    brains = []
    do_apply = True
    # this will add a specific class to the generated button action
    # so it is possible to skin it with an icon
    button_with_icon = False
    overlay = True

    def available(self):
        """Will the action be available for current context?"""
        return True

    def _update(self):
        """Method to override if you need to do something in the update."""
        return

    def _update_widgets(self):
        """Method to override if you need to do something after the updateWidgets method."""
        return

    def _apply(self, **data):
        """This method receives in data the form content and does the apply logic.
           It is the method to implement if default handleApply is enough."""
        raise NotImplementedError

    def update(self):
        form = self.request.form
        if 'form.widgets.uids' in form:
            uids = form['form.widgets.uids']
        else:
            uids = self.request.get('uids', '')
            form['form.widgets.uids'] = uids

        if 'form.widgets.referer' not in form:
            form['form.widgets.referer'] = self.request.get('referer', '').replace('@', '&').replace('!', '#')

        self.brains = self.brains or brains_from_uids(uids)

        # sort buttons
        self._old_buttons = self.buttons
        self.buttons = self.buttons.select('apply', 'cancel')
        self._update()
        super(BaseBatchActionForm, self).update()
        self._update_widgets()

    @button.buttonAndHandler(_(u'Apply'), name='apply', condition=lambda fi: fi.do_apply)
    def handleApply(self, action):
        """ """
        if not self.available():
            raise Unauthorized

        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
        else:
            # call the method that does the job
            self._apply(**data)
            self.request.response.redirect(self.request.form['form.widgets.referer'])

    @button.buttonAndHandler(PMF(u'Cancel'), name='cancel')
    def handleCancel(self, action):
        self.request.response.redirect(self.request.get('HTTP_REFERER'))


def brains_from_uids(uids):
    """ Returns a list of brains from a string containing uids separated by comma """
    uids = uids.split(',')
    catalog = api.portal.get_tool('portal_catalog')
    brains = catalog(UID=uids)
    return brains


class TransitionBatchActionForm(BaseBatchActionForm):

    label = _(u"Batch state change")

    def getAvailableTransitionsVoc(self):
        """ Returns available transitions common for all brains """
        wtool = api.portal.get_tool(name='portal_workflow')
        terms = []
        transitions = None
        for brain in self.brains:
            obj = brain.getObject()
            if transitions is None:
                transitions = set([(tr['id'], tr['title']) for tr in wtool.getTransitionsFor(obj)])
            else:
                transitions &= set([(tr['id'], tr['title']) for tr in wtool.getTransitionsFor(obj)])
        if transitions:
            for (id, tit) in transitions:
                terms.append(
                    SimpleTerm(id,
                               id,
                               translate(tit,
                                         domain='plone',
                                         context=self.request)))
        terms = sorted(terms, key=attrgetter('title'))
        return SimpleVocabulary(terms)

    def _update(self):
        self.voc = self.getAvailableTransitionsVoc()
        self.do_apply = len(self.voc) > 0
        self.fields += Fields(schema.Choice(
            __name__='transition',
            title=_(u'Transition'),
            vocabulary=self.voc,
            description=(len(self.voc) == 0 and
                         _(u'No common or available transition. Modify your selection.') or u''),
            required=len(self.voc) > 0))
        self.fields += Fields(schema.Text(
            __name__='comment',
            title=_(u'Comment'),
            description=_(u'Optional comment to display in history'),
            required=False))

    def _apply(self, **data):
        """ """
        if data['transition']:
            for brain in self.brains:
                obj = brain.getObject()
                api.content.transition(obj=obj,
                                       transition=data['transition'],
                                       comment=data['comment'])



# -*- coding: utf-8 -*-

from zope.component import getMultiAdapter
from collective.eeafaceted.batchactions.tests.base import BaseTestCase
from plone import api


class TestActions(BaseTestCase):

    def setUp(self):
        """ """
        super(TestActions, self).setUp()
        self.doc1 = api.content.create(
            type='Document',
            id='doc1',
            title='Document 1',
            container=self.portal
        )
        self.doc2 = api.content.create(
            type='Document',
            id='doc2',
            title='Document 2',
            container=self.portal
        )

    def test_transition_action_apply(self):
        """Working behavior, we have several documents with same transition available."""
        # set 'uids' in form
        doc_uids = u"{0},{1}".format(self.doc1.UID(), self.doc2.UID())
        self.request.form['form.widgets.uids'] = doc_uids
        form = self.eea_folder.restrictedTraverse('transition-batch-action')
        # common transitions are shown, here it is the case as docs are in same state
        form.update()
        self.assertEqual(
            form.widgets['transition'].terms.terms.by_token.keys(),
            ['publish', 'submit'])
        form.request['form.widgets.transition'] = 'publish'
        extracted_data, errors = form.extractData()
        self.assertEqual(
            extracted_data,
            {'comment': None,
             'transition': 'publish',
             'referer': None,
             'uids': doc_uids}, ())

        # for now both docs are 'private'
        self.assertEqual(
            [api.content.get_state(self.doc1), api.content.get_state(self.doc2)],
            ['private', 'private'])
        form.handleApply(form, None)
        self.assertEqual(
            [api.content.get_state(self.doc1), api.content.get_state(self.doc2)],
            ['published', 'published'])

    def test_transition_action_cancel(self):
        """When cancelled, nothing is done and user is redirected to referer."""
        form = getMultiAdapter((self.eea_folder, self.request), name='transition-batch-action')
        self.request['HTTP_REFERER'] = self.portal.absolute_url()
        self.request.RESPONSE.status = 200
        self.assertNotEqual(
            self.request.RESPONSE.getHeader('location'),
            self.request['HTTP_REFERER'])
        form.handleCancel(form, None)
        self.assertEqual(self.request.RESPONSE.status, 302)
        self.assertEqual(
            self.request.RESPONSE.getHeader('location'),
            self.request['HTTP_REFERER'])

    def test_transition_action_uids_can_be_defined_on_request_or_form(self):
        """'uids' used by the form are retrieved no matter it is defined on
            self.request or self.request.form."""
        # set 'uids' in self.request.form
        doc_uids = u"{0},{1}".format(self.doc1.UID(), self.doc2.UID())
        self.request.form['form.widgets.uids'] = doc_uids
        form = self.eea_folder.restrictedTraverse('transition-batch-action')
        # common transitions are shown, here it is the case as docs are in same state
        form.update()
        extracted_data, errors = form.extractData()
        self.assertEqual(
            extracted_data['uids'], doc_uids)
        del self.request.form['form.widgets.uids']

        # set 'uids' in self.request
        doc_uids = u"{0},{1}".format(self.doc1.UID(), self.doc2.UID())
        self.request['uids'] = doc_uids
        form = self.eea_folder.restrictedTraverse('transition-batch-action')
        # common transitions are shown, here it is the case as docs are in same state
        form.update()
        extracted_data, errors = form.extractData()
        self.assertEqual(
            extracted_data['uids'], doc_uids)

    def test_transition_action_only_list_common_transitions(self):
        """Only work if there are common transitions for selected elements."""
        # set 'uids' in form
        doc_uids = u"{0},{1}".format(self.doc1.UID(), self.doc2.UID())
        self.request.form['form.widgets.uids'] = doc_uids
        form = self.eea_folder.restrictedTraverse('transition-batch-action')
        # common transitions are shown, here it is the case as docs are in same state
        form.update()
        self.assertEqual(
            form.widgets['transition'].terms.terms.by_token.keys(),
            ['publish', 'submit'])

        # change state of doc1, no common transition available
        api.content.transition(self.doc1, 'publish')
        form = self.eea_folder.restrictedTraverse('transition-batch-action')
        form.update()
        self.assertEqual(
            form.widgets['transition'].terms.terms.by_token.keys(),
            [])

        # only one selected element
        # doc1
        doc_uids = u"{0}".format(self.doc1.UID())
        self.request.form['form.widgets.uids'] = doc_uids
        form = self.eea_folder.restrictedTraverse('transition-batch-action')
        form.update()
        self.assertEqual(
            form.widgets['transition'].terms.terms.by_token.keys(),
            ['retract', 'reject'])
        # doc2
        doc_uids = u"{0}".format(self.doc2.UID())
        self.request.form['form.widgets.uids'] = doc_uids
        form = self.eea_folder.restrictedTraverse('transition-batch-action')
        form.update()
        self.assertEqual(
            form.widgets['transition'].terms.terms.by_token.keys(),
            ['publish', 'submit'])

    def test_transition_action_button_visibility(self):
        """Button 'Apply' is only shown if there are common transitions."""
        doc_uids = u"{0},{1}".format(self.doc1.UID(), self.doc2.UID())
        self.request.form['form.widgets.uids'] = doc_uids
        form = self.eea_folder.restrictedTraverse('transition-batch-action')
        # button is shown as there are common transitions
        form.update()
        self.assertTrue(form.widgets['transition'].terms.terms.by_token.keys())
        apply_button = form.buttons.get('apply')
        self.assertTrue(bool(apply_button.condition(form)))

        # change state of doc1, no common transition available
        api.content.transition(self.doc1, 'publish')
        form = self.eea_folder.restrictedTraverse('transition-batch-action')
        form.update()
        self.assertFalse(bool(apply_button.condition(form)))



# -*- coding: utf-8 -*-
from collective.eeafaceted.batchactions.interfaces import IBatchActionsMarker


class IBatchActionsSpecificMarker(IBatchActionsMarker):
    """A marker that is more specific thatn IBatchActionsMarker."""



# -*- coding: utf-8 -*-

import unittest
from collective.eeafaceted.batchactions import testing
from collective.eeafaceted.batchactions.interfaces import IBatchActionsMarker
from eea.facetednavigation.layout.interfaces import IFacetedLayout
from plone import api
from zope.interface import alsoProvides


class BaseTestCase(unittest.TestCase):

    layer = testing.FUNCTIONAL

    def setUp(self):
        """ """
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        # create a folder and enable faceted navigation on it
        eea_folder = api.content.create(
            type='Folder',
            id='eea_folder',
            title='EEA Folder',
            container=self.portal
        )
        alsoProvides(eea_folder, IBatchActionsMarker)
        eea_folder.reindexObject()
        eea_folder.restrictedTraverse('@@faceted_subtyper').enable()
        IFacetedLayout(eea_folder).update_layout('faceted-table-items')
        self.eea_folder = eea_folder



from collective.eeafaceted.batchactions.browser.views import BaseBatchActionForm


class TestingBatchActionForm(BaseBatchActionForm):

    buttons = BaseBatchActionForm.buttons.copy()
    label = (u"Testing form")
    button_with_icon = True
    overlay = False

    def available(self):
        """Available if 'hide_testing_action' not found in request."""
        res = super(TestingBatchActionForm, self).available()
        if res and not self.request.get('hide_testing_action'):
            return True
        return False



from zope.publisher.browser import BrowserView
from zope.i18n import translate as _

TEMPLATE = """\
var no_selected_items = "%(no_selected_items)s";
"""


class JSVariables(BrowserView):

    def __call__(self, *args, **kwargs):
        response = self.request.response
        response.setHeader('content-type', 'text/javascript;;charset=utf-8')

        no_selected_items = _('no_selected_items',
                              default="Please select at least one element.",
                              domain='collective.eeafaceted.batchactions',
                              context=self.request)
        return TEMPLATE % dict(
            no_selected_items=no_selected_items,
        )



# -*- coding: utf-8 -*-
"""Installer for the collective.eeafaceted.batchactions package."""

from setuptools import find_packages
from setuptools import setup

import os


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

long_description = \
    read('README.rst') + read('CHANGES.rst')

setup(
    name='collective.eeafaceted.batchactions',
    version='0.8',
    description="This package provides batch actions for eea.facetednavigation dashboard",
    long_description=long_description,
    # Get more from http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Plone :: 4.3",
        "Programming Language :: Python :: 2.7",
    ],
    keywords='Python Zope Plone',
    author='IMIO',
    author_email='dev@imio.be',
    url='http://pypi.python.org/pypi/collective.eeafaceted.batchactions',
    license='GPL V2',
    packages=find_packages('src', exclude=['ez_setup']),
    namespace_packages=['collective', 'collective.eeafaceted'],
    package_dir={'': 'src'},
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'Plone',
        'setuptools',
        'collective.eeafaceted.z3ctable',
        ],
    extras_require={
        'test': [
            'plone.app.dexterity',
            'plone.app.testing',
            'plone.app.relationfield',
            'plone.app.robotframework',
        ],
        'develop': [
            'zest.releaser',
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: utf-8 -*-

from AccessControl import Unauthorized
from collective.eeafaceted.batchactions.interfaces import IBatchActionsMarker
from collective.eeafaceted.batchactions.tests.base import BaseTestCase
from collective.eeafaceted.batchactions.tests.interfaces import IBatchActionsSpecificMarker
from plone import api
from Products.Five.browser import BrowserView
from zope.component import getMultiAdapter
from zope.interface import alsoProvides
from zope.viewlet.interfaces import IViewletManager


class TestViewlets(BaseTestCase):

    def _get_viewlet_manager(self, context):
        """ """
        view = BrowserView(self.eea_folder, self.request)
        manager_name = 'collective.eeafaceted.z3ctable.bottomabovenav'
        viewlet_manager = getMultiAdapter(
            (context, self.request, view),
            IViewletManager,
            manager_name)
        viewlet_manager.update()
        return viewlet_manager

    def _get_viewlet(self, context):
        """ """
        viewlet_manager = self._get_viewlet_manager(context)
        viewlet = viewlet_manager.get(u'collective.eeafaceted.batchactions')
        return viewlet

    def test_viewlet_available(self):
        """Available by default."""
        viewlet = self._get_viewlet(self.eea_folder)
        self.assertTrue(viewlet.available())

    def test_viewlet_only_rendered_on_IBatchActionsMarker(self):
        """ """
        folder = api.content.create(
            type='Folder',
            id='folder',
            title='Folder',
            container=self.portal
        )
        viewlet = self._get_viewlet(folder)
        self.assertIsNone(viewlet)
        alsoProvides(folder, IBatchActionsMarker)
        viewlet = self._get_viewlet(folder)
        self.assertEqual(viewlet.__name__, u'collective.eeafaceted.batchactions')

    def test_get_marker_interfaces(self):
        """_get_marker_interfaces will return the marker interfaces views
           may be registered for.  It is the IBatchActionsMarker and others
           inheriting from it."""
        viewlet = self._get_viewlet(self.eea_folder)
        # the u'collective.eeafaceted.batchactions' viewlet exists in the viewlet manager
        # as IBatchActionsMarker is implemented by self.eea_folder
        self.assertEqual(viewlet.__name__, u'collective.eeafaceted.batchactions')
        self.assertEqual(viewlet._get_marker_interfaces(), [IBatchActionsMarker])

        # if an interface suclassing IBatchActionsMarker is found, it is also returned
        alsoProvides(self.eea_folder, IBatchActionsSpecificMarker)
        self.assertEqual(
            viewlet._get_marker_interfaces(),
            [IBatchActionsMarker, IBatchActionsSpecificMarker])

    def test_get_batch_actions(self):
        """This will return every found action names.
           We test here classical functionnality with actions registered for IBatchActionsMarker."""
        viewlet = self._get_viewlet(self.eea_folder)
        self.assertEqual(
            viewlet.get_batch_actions(),
            [{'name': 'transition-batch-action', 'button_with_icon': False, 'overlay': True}])

        # returned action names are traversable to get the form
        for action in viewlet.get_batch_actions():
            form = self.eea_folder.restrictedTraverse(action['name'])
            self.assertEqual(form.__name__, action['name'])

    def test_get_batch_actions_available(self):
        """A method 'available' is evaluated on the action view to check if it is available on context."""
        # mark eea_folder with IBatchActionsSpecificMarker so testing-batch-action is useable
        alsoProvides(self.eea_folder, IBatchActionsSpecificMarker)
        viewlet = self._get_viewlet(self.eea_folder)
        self.assertTrue('testing-batch-action' in [action['name'] for action in viewlet.get_batch_actions()])
        # 'testing-batch-action' is available if value 'hide_testing_action' not found in request
        self.request.set('hide_testing_action', True)
        self.assertFalse('testing-batch-action' in [action['name'] for action in viewlet.get_batch_actions()])
        # trying to execute a not available action will raise Unauthorized
        testing_form = getMultiAdapter((self.eea_folder, self.request), name=u'testing-batch-action')
        self.assertRaises(Unauthorized, testing_form.handleApply, testing_form, None)

    def test_get_batch_actions_consider_new_action_specific_interface(self):
        """Register a view for IBatchActionsSpecificMarker, get_batch_actions will
           behave differently if context implementing interface or not."""
        folder = api.content.create(
            type='Folder',
            id='folder',
            title='Folder',
            container=self.portal
        )
        alsoProvides(folder, IBatchActionsMarker)
        viewlet = self._get_viewlet(folder)
        self.assertEqual(
            viewlet.get_batch_actions(),
            [{'name': 'transition-batch-action', 'button_with_icon': False, 'overlay': True}])

        # mark with IBatchActionsSpecificMarker
        alsoProvides(folder, IBatchActionsSpecificMarker)
        self.assertEqual(
            viewlet.get_batch_actions(),
            [{'name': 'testing-batch-action', 'button_with_icon': True, 'overlay': False},
             {'name': 'transition-batch-action', 'button_with_icon': False, 'overlay': True}])
        # returned action names are traversable to get the form
        for action in viewlet.get_batch_actions():
            form = folder.restrictedTraverse(action['name'])
            self.assertEqual(form.__name__, action['name'])

        # still correct on eea_folder that does not implements IBatchActionsSpecificMarker
        viewlet = self._get_viewlet(self.eea_folder)
        self.assertEqual(
            viewlet.get_batch_actions(),
            [{'name': 'transition-batch-action', 'button_with_icon': False, 'overlay': True}])

    def test_js_variables(self):
        """Some JS variables are defined for translation purpose."""
        js_variables = self.portal.restrictedTraverse(
            'collective_eeafaceted_batchactions_js_variables.js')
        self.assertEqual(
            js_variables(),
            'var no_selected_items = "Please select at least one element.";\n')
